package TKL::File;

use Data::Dumper;
use TKL;
use XML::LibXSLT;
use Time::Local;
use strict;
use vars qw(@ISA);

@ISA = qw(TKL);


sub filename {  ## absolute path to the file
    return shift->{file};
}


sub portal_filename {
    my $self = shift;
    my $abs = $self->filename;
    my $root = $self->find_portal_root;
    
    $root =~ s/\//\\\//g;	## Replace / by \/
    $abs =~ s/^$root//;		## Remove the root part of the filename
    return $abs;
}

sub file_exists {
    my $self = shift;
    return ( -f $self->filename);
}

sub file_writable {
    my $self = shift;
    return ( -w $self->filename);
}


sub is_symlink {
    my $self = shift;
    return ( -l $self->filename);
}

   
sub get_timestamp {
## Returns timestamp for the record, currently you can extract
## timestamps for modified and created fields.

    my ($self, $which) = @_;
    my $doc_elem = $self->document_element;
    my $timestamp = $doc_elem->getAttribute($which); 

    if (defined($timestamp)) {
	my ($year, $month, $day, $hour, $min, $sec) = ($timestamp =~ /(\d+)-(\d+)-(\d+), (\d+):(\d+):(\d+)/);
	return timelocal($sec, $min, $hour, $day, $month - 1, $year);
    } else {
	return undef;
    }
}
    


sub render {
    my ($self) = @_;
    my $doc_elem = $self->document_element;
    my $parser = $self->dom;
    my $xslt = new XML::LibXSLT;
    my $doc_elem_name = $doc_elem->nodeName();
    my $file = $self->filename;

    $file =~ s/[^\/]*$//;
    $file .= "$doc_elem_name.xsl";

    my $template = $self->locate_file($file);
    if (defined($template)) {
	my $style_doc = $parser->parse_file($template);
	my $stylesheet = $xslt->parse_stylesheet($style_doc);
	my $result = $stylesheet->transform($doc_elem->ownerDocument);
	return $stylesheet->output_string($result);
    } else {
	return $doc_elem->toString;
    }
}
    

sub get_base {
    my $self = shift;
    my $file = $self->filename;
    my @path = split(/\//, $file);
    
    return pop @path;
}


sub extract_schema {
    my $self = shift;
    my $file = $self->filename;

    my $root = $self->document_element($file);
    my $element_name = $root->nodeName();

    return "$element_name.$TKL::schema_suffix";
}



sub document_element {
    my $self = shift;

    unless (ref($self->{document_element}) eq "XML::LibXML::Node") {
	my $doc_elem = $self->SUPER::document_element($self->filename);
	$self->{document_element} = $doc_elem;
    }

    return $self->{document_element};
}


sub parse {
    my $self = shift;

    return $self->SUPER::parse($self->filename);
}

# Writes new content to the file
sub storefile {
    my $self = shift;
    my $dom = shift;
    # TODO: Should we update the modified attribute
    #       or is that for human modifications only?
    my $fn=$self->{file};
    my $xml= $dom->toString();
    open F, ">".$fn or croak("Could not open $fn for writing");
    print F $TKL::xml_header, "\n";
    print F $xml,"\n";
    close(F);

}

1;

__END__

=head1 NAME

TKL::File - Perl package implementing simple methods for accessing a TKL file.

=head1 SYNOPSIS

  use TKL::File;

  my $tkl = new TKL::File(filename => 'myfile.tkl');

=head1 DESCRIPTION

=head1 AUTHOR

Anders Snderberg Mortensen <sondberg@indexdata.dk>
Indexdata
2003/06/12

=head1 SEE-ALSO

perl(1).

=cut

## $Log: File.pm,v $
## Revision 1.1  2003/08/04 13:32:56  marc
## added perl code
##
## Revision 1.7  2003/07/31 12:06:57  sondberg
## The statistical reports neglects symlinks.
##
## Revision 1.6  2003/07/05 22:07:06  sondberg
## Started on implementing the render method - intended to invoke XSLT processor
## and return formatted HTML.
##
## Revision 1.5  2003/06/30 15:09:49  heikki
## Most of task.pm stuff now in right place.
##
## Revision 1.4  2003/06/17 13:11:48  sondberg
## Added this library to handle interaction with users.tkl.
##
## Revision 1.3  2003/06/13 11:11:01  sondberg
## Added method portal_filename.
##
## Revision 1.2  2003/06/12 12:31:44  sondberg
## Added some POD (just a starting-point).
##
## Revision 1.1  2003/06/12 11:30:22  sondberg
## Moved around.
##
## Revision 1.4  2003/05/13 19:14:35  heikki
## Working on get_task.pl
## Recurses through all task files, and changes status as needed.
## Does not yet start actual harvesting
##
## Revision 1.3  2002/12/10 11:18:02  sondberg
## Added method get_base.
##
## Revision 1.2  2002/11/28 21:42:47  sondberg
## Overrides generic parser method.
##
## Revision 1.1  2002/11/27 14:07:38  sondberg
## Added these files to repository.
##
